const CLASS_PART_SEPARATOR = '-';
export function createClassUtils(config) {
    const classMap = {
        nextPart: {},
        validators: [],
    };
    processStandaloneClasses(config, classMap);
    const processDynamicClasses = createProcessDynamicClasses(config, classMap);
    function getGroup(className) {
        const classParts = className.split(CLASS_PART_SEPARATOR);
        // Classes like `-inset-1` produce an empty string as first classPart. We assume that classes for negative values are used correctly and remove it from classParts.
        if (classParts[0] === '' && classParts.length > 1) {
            classParts.shift();
        }
        processDynamicClasses(classParts[0]);
        return getGroupRecursive(classParts, classMap);
    }
    function getConflictingGroups(classGroupId) {
        return config.conflictingGroups[classGroupId] || [];
    }
    return {
        getGroup,
        getConflictingGroups,
    };
}
function getGroupRecursive(classParts, classPartObject) {
    if (classParts.length === 0) {
        return classPartObject.classGroupId;
    }
    const currentClassPart = classParts[0];
    const nextClassPartObject = classPartObject.nextPart[currentClassPart];
    const classGroupFromNextClassPart = nextClassPartObject
        ? getGroupRecursive(classParts.slice(1), nextClassPartObject)
        : undefined;
    if (classGroupFromNextClassPart) {
        return classGroupFromNextClassPart;
    }
    if (classPartObject.validators.length === 0) {
        return undefined;
    }
    const classRest = classParts.join(CLASS_PART_SEPARATOR);
    return classPartObject.validators.find(({ validator }) => validator(classRest))?.classGroupId;
}
function processStandaloneClasses(config, classMap) {
    config.standaloneClasses.forEach((classGroup, index) => {
        const classGroupId = `standaloneClasses.${index}`;
        classGroup.forEach((className) => {
            addToClassPart({
                classPartObject: classMap,
                path: className,
                classGroupId,
            });
        });
    });
}
function createProcessDynamicClasses(config, classMap) {
    const { dynamicClasses } = config;
    const processedKeys = new Set();
    return function processDynamicClasses(key) {
        if (processedKeys.has(key) || dynamicClasses[key] === undefined) {
            return;
        }
        const classPartObject = getNextPart(classMap, key);
        dynamicClasses[key].forEach((classGroup, index) => {
            processClassesRecursively(classGroup, classPartObject, `dynamicClasses.${key}.${index}`);
        });
    };
}
function processClassesRecursively(classGroup, classPartObject, classGroupId) {
    classGroup.forEach((classDefinition) => {
        if (typeof classDefinition === 'string') {
            addToClassPart({
                classPartObject,
                // We default to undefined to set classGroupId on current classPartObject when classDefinition === ''
                path: classDefinition === '' ? undefined : classDefinition,
                classGroupId,
            });
        }
        else if (typeof classDefinition === 'function') {
            addToClassPart({
                classPartObject,
                validator: {
                    validator: classDefinition,
                    classGroupId,
                },
            });
        }
        else {
            Object.entries(classDefinition).forEach(([key, classGroup]) => {
                processClassesRecursively(classGroup, getNextPart(classPartObject, key), classGroupId);
            });
        }
    });
}
function addToClassPart({ path, classPartObject, validator, classGroupId }) {
    let currentClassPartObject = classPartObject;
    path?.split(CLASS_PART_SEPARATOR).forEach((pathPart) => {
        currentClassPartObject = getNextPart(currentClassPartObject, pathPart);
    });
    if (classGroupId) {
        currentClassPartObject.classGroupId = classGroupId;
    }
    if (validator) {
        currentClassPartObject.validators.push(validator);
    }
}
function getNextPart(classPartObject, nextPart) {
    if (classPartObject.nextPart[nextPart] === undefined) {
        classPartObject.nextPart[nextPart] = {
            nextPart: {},
            validators: [],
        };
    }
    return classPartObject.nextPart[nextPart];
}
//# sourceMappingURL=class-utils.js.map